/**********************************************************************************
   Copyright(c) Catalyst Semiconductor, Inc., 2005. All rights reserved.

This code is provided as is without warranty or guarantee of any kind as its accuracy, 
completeness, operability, fitness for particular purpose, or any other warranty.

   File        :  MWmaster.v
   Description :  Verilog model for Microwire protocol - master side
		  for testing Verilog model for CAT93XXX Microwire serial EEPROMs
   Version     :  Rev1.00 
   Release     :  Jan, 2005
**********************************************************************************/

`timescale 1ns/1ns
//
//	Master for Microwire protocol
//
module MWmaster(SI,SO,SCK,CSB,ORG,PE);
input SO;
tri		SO;
output 	SI,SCK,CSB,ORG,PE;
reg 		SI,SCK,CSB,ORG,PE;
	
`include "MemType.v"

`ifdef cat93c46
	`include "Timing_46_56_57_66.v"
`endif
`ifdef cat93hc46
	`include "Timing_46_56_57_66.v"
`endif
`ifdef cat93c56
	`include "Timing_46_56_57_66.v"
`endif
`ifdef cat93c57
	`include "Timing_46_56_57_66.v"
`endif
`ifdef cat93c66
	`include "Timing_46_56_57_66.v"
`endif
`ifdef cat93c76
	`include "Timing_76.v"
`endif
`ifdef cat93c86
	`include "Timing_86.v"
`endif

`define B0			1'b0
`define B1			1'b1

parameter frequency = 250000; //(Hz)
parameter dutyFactor = 1.0/2.0;
//
//	states
//
`define idle 				4'b0000
`define sendingInstruction 	4'b0001
`define sendingData			4'b1100
`define sendingStopStart		4'b1011
`define receivingData		4'b0111
`define internalWrite		4'b1111

reg[13:0] bufEWEN, bufEWDS, bufREAD, bufWRITE, bufWRAL, bufERASE, bufERAL;

reg[2:0] EWEN 	= 3'b000;
reg[2:0] EWDS	= 3'b001;
reg[2:0] READ	= 3'b010;
reg[2:0] WRITE	= 3'b011;
reg[2:0] WRAL	= 3'b100;
reg[2:0] ERASE	= 3'b101;
reg[2:0] ERAL	= 3'b110;

reg StartBit = 1'b1;

reg[1:0] OpCodeEWEN 	= 2'b00;  
reg[1:0] OpCodeEWDS 	= 2'b00;  
reg[1:0] OpCodeREAD 	= 2'b10;  
reg[1:0] OpCodeWRITE 	= 2'b01;  
reg[1:0] OpCodeWRAL 	= 2'b00;  
reg[1:0] OpCodeERASE 	= 2'b11;  
reg[1:0] OpCodeERAL 	= 2'b00;
  
reg[6:0] AdrAny8_4 	= 7'b0000000;
reg[5:0] AdrAny16_4 	= 6'b000000;
reg[8:0] AdrAny8_5 	= 9'b000000000;
reg[7:0] AdrAny16_5 	= 8'b00000000;
reg[8:0] AdrAny8_6 	= 9'b000000000;
reg[7:0] AdrAny16_6 	= 8'b00000000;
reg[7:0] AdrAny8_7 	= 8'b00000000;
reg[6:0] AdrAny16_7 	= 7'b0000000;
reg[10:0] AdrAny8_8 	= 11'b00000000000;
reg[9:0] AdrAny16_8 	= 10'b0000000000;

reg[6:0] AdrEWEN8_4, AdrEWDS8_4, AdrERAL8_4, AdrWRAL8_4;
reg[5:0] AdrEWEN16_4, AdrEWDS16_4, AdrERAL16_4, AdrWRAL16_4;
reg[8:0] AdrEWEN8_5, AdrEWDS8_5, AdrERAL8_5, AdrWRAL8_5;
reg[7:0] AdrEWEN16_5, AdrEWDS16_5, AdrERAL16_5, AdrWRAL16_5; 
reg[8:0] AdrEWEN8_6, AdrEWDS8_6, AdrERAL8_6, AdrWRAL8_6;
reg[7:0] AdrEWEN16_6, AdrEWDS16_6, AdrERAL16_6, AdrWRAL16_6;  
reg[7:0] AdrEWEN8_7, AdrEWDS8_7, AdrERAL8_7, AdrWRAL8_7;
reg[6:0] AdrEWEN16_7, AdrEWDS16_7, AdrERAL16_7, AdrWRAL16_7;  
reg[10:0] AdrEWEN8_8, AdrEWDS8_8, AdrERAL8_8, AdrWRAL8_8;
reg[9:0] AdrEWEN16_8, AdrEWDS16_8, AdrERAL16_8, AdrWRAL16_8;  

reg[9:0] EWEN8_4, EWDS8_4, READ8_4, WRITE8_4, WRAL8_4, ERASE8_4, ERAL8_4;
reg[8:0] EWEN16_4, EWDS16_4, READ16_4, WRITE16_4, WRAL16_4, ERASE16_4, ERAL16_4;
reg[11:0] EWEN8_5, EWDS8_5, READ8_5, WRITE8_5, WRAL8_5, ERASE8_5, ERAL8_5;
reg[10:0] EWEN16_5, EWDS16_5, READ16_5, WRITE16_5, WRAL16_5, ERASE16_5, ERAL16_5;
reg[11:0] EWEN8_6, EWDS8_6, READ8_6, WRITE8_6, WRAL8_6, ERASE8_6, ERAL8_6;
reg[10:0] EWEN16_6, EWDS16_6, READ16_6, WRITE16_6, WRAL16_6, ERASE16_6, ERAL16_6;
reg[10:0] EWEN8_7, EWDS8_7, READ8_7, WRITE8_7, WRAL8_7, ERASE8_7, ERAL8_7;
reg[9:0] EWEN16_7, EWDS16_7, READ16_7, WRITE16_7, WRAL16_7, ERASE16_7, ERAL16_7;
reg[13:0] EWEN8_8, EWDS8_8, READ8_8, WRITE8_8, WRAL8_8, ERASE8_8, ERAL8_8;
reg[12:0] EWEN16_8, EWDS16_8, READ16_8, WRITE16_8, WRAL16_8, ERASE16_8, ERAL16_8;
//
//	input parameters from test sequence ("Input.v" file)
//
reg[7:0] Minstruction, dataByte;	
reg[15:0] addr, dataBytes;
reg [3:0] pattern;
integer lSeq, lgMem;	
integer tHALF,tDelayIni,delta;					
//
//	work variables
//
integer realLSeq;						
integer MbitCounter, seqCounter, i, j, recDataCounter, lgDisp;
integer TestPhase;
reg [13:0] MtempReg,manRegB; 
reg [22:0] manRegW;
reg [15:0] manAddr;

reg [7:0] bufWriteData[0:2047];		
reg [7:0] bufReadData[0:2047];
reg [3:0] genDataCell;
reg [7:0] linestr[0:79];

reg [3:0] Mstate,MnextState; 			// State machine controller

//
//###################### INITIAL #############################
//

initial
	begin
	delta = 1;
	PE = 0;
	`ifdef PEset
		PE = 1;
	`endif

	$display("*");
	$display("*    Master: clock frequency = %d Hz",frequency);
	$display("*    Master: duty factor     = %g ",dutyFactor);

	tHALF = 1000000000/(2*frequency);      // Half Clock Period (in ns)
	tDelayIni = 1000;

AdrEWEN8_4 	= {2'b11,5'b00000};
AdrEWEN16_4 	= {2'b11,4'b0000};
AdrEWEN8_5 	= {2'b11,7'b0000000};
AdrEWEN16_5 	= {2'b11,6'b000000};
AdrEWEN8_6 	= {2'b11,7'b0000000};
AdrEWEN16_6 	= {2'b11,6'b000000};
AdrEWEN8_7 	= {2'b11,6'b000000};
AdrEWEN16_7 	= {2'b11,5'b00000};
AdrEWEN8_8 	= {2'b11,9'b000000000};
AdrEWEN16_8 	= {2'b11,8'b00000000};

AdrEWDS8_4 	= {2'b00,5'b00000};
AdrEWDS16_4 	= {2'b00,4'b0000};
AdrEWDS8_5 	= {2'b00,7'b0000000};
AdrEWDS16_5 	= {2'b00,6'b000000};
AdrEWDS8_6 	= {2'b00,7'b0000000};
AdrEWDS16_6 	= {2'b00,6'b000000};
AdrEWDS8_7 	= {2'b00,6'b000000};
AdrEWDS16_7 	= {2'b00,5'b00000};
AdrEWDS8_8 	= {2'b00,9'b000000000};
AdrEWDS16_8 	= {2'b00,8'b00000000};

AdrERAL8_4 	= {2'b10,5'b00000};
AdrERAL16_4 	= {2'b10,4'b0000};
AdrERAL8_5 	= {2'b10,7'b0000000};
AdrERAL16_5 	= {2'b10,6'b000000};
AdrERAL8_6 	= {2'b10,7'b0000000};
AdrERAL16_6 	= {2'b10,6'b000000};
AdrERAL8_7 	= {2'b10,6'b000000};
AdrERAL16_7 	= {2'b10,5'b00000};
AdrERAL8_8 	= {2'b10,9'b000000000};
AdrERAL16_8 	= {2'b10,8'b00000000};

AdrWRAL8_4 	= {2'b01,5'b00000};
AdrWRAL16_4 	= {2'b01,4'b0000};
AdrWRAL8_5 	= {2'b01,7'b0000000};
AdrWRAL16_5 	= {2'b01,6'b000000};
AdrWRAL8_6 	= {2'b01,7'b0000000};
AdrWRAL16_6 	= {2'b01,6'b000000};
AdrWRAL8_7 	= {2'b01,6'b000000};
AdrWRAL16_7 	= {2'b01,5'b00000};
AdrWRAL8_8 	= {2'b01,9'b000000000};
AdrWRAL16_8 	= {2'b01,8'b00000000};

EWEN8_4 	= {StartBit,OpCodeEWEN,AdrEWEN8_4};		// Enable Write
EWEN16_4 	= {StartBit,OpCodeEWEN,AdrEWEN16_4};
EWEN8_5 	= {StartBit,OpCodeEWEN,AdrEWEN8_5};
EWEN16_5 	= {StartBit,OpCodeEWEN,AdrEWEN16_5};
EWEN8_6 	= {StartBit,OpCodeEWEN,AdrEWEN8_6};
EWEN16_6 	= {StartBit,OpCodeEWEN,AdrEWEN16_6};
EWEN8_7 	= {StartBit,OpCodeEWEN,AdrEWEN8_7};
EWEN16_7 	= {StartBit,OpCodeEWEN,AdrEWEN16_7};
EWEN8_8 	= {StartBit,OpCodeEWEN,AdrEWEN8_8};
EWEN16_8 	= {StartBit,OpCodeEWEN,AdrEWEN16_8};

EWDS8_4 	= {StartBit,OpCodeEWDS,AdrEWDS8_4};		// Disable Write
EWDS16_4 	= {StartBit,OpCodeEWDS,AdrEWDS16_4};
EWDS8_5 	= {StartBit,OpCodeEWDS,AdrEWDS8_5};  
EWDS16_5 	= {StartBit,OpCodeEWDS,AdrEWDS16_5};
EWDS8_6 	= {StartBit,OpCodeEWDS,AdrEWDS8_6};  
EWDS16_6 	= {StartBit,OpCodeEWDS,AdrEWDS16_6};
EWDS8_7 	= {StartBit,OpCodeEWDS,AdrEWDS8_7};  
EWDS16_7 	= {StartBit,OpCodeEWDS,AdrEWDS16_7};
EWDS8_8 	= {StartBit,OpCodeEWDS,AdrEWDS8_8};  
EWDS16_8 	= {StartBit,OpCodeEWDS,AdrEWDS16_8};

READ8_4 	= {StartBit,OpCodeREAD,AdrAny8_4};  		// Read Data from Memory
READ16_4 	= {StartBit,OpCodeREAD,AdrAny16_4};   
READ8_5 	= {StartBit,OpCodeREAD,AdrAny8_5};  
READ16_5 	= {StartBit,OpCodeREAD,AdrAny16_5}; 
READ8_6 	= {StartBit,OpCodeREAD,AdrAny8_6};  
READ16_6 	= {StartBit,OpCodeREAD,AdrAny16_6};  
READ8_7 	= {StartBit,OpCodeREAD,AdrAny8_7};  
READ16_7 	= {StartBit,OpCodeREAD,AdrAny16_7};   
READ8_8 	= {StartBit,OpCodeREAD,AdrAny8_8};  
READ16_8 	= {StartBit,OpCodeREAD,AdrAny16_8};  

WRITE8_4 	= {StartBit,OpCodeWRITE,AdrAny8_4};  		// Write Data to Memory
WRITE16_4 	= {StartBit,OpCodeWRITE,AdrAny16_4};  
WRITE8_5 	= {StartBit,OpCodeWRITE,AdrAny8_5};  
WRITE16_5 	= {StartBit,OpCodeWRITE,AdrAny16_5}; 
WRITE8_6 	= {StartBit,OpCodeWRITE,AdrAny8_6};  
WRITE16_6 	= {StartBit,OpCodeWRITE,AdrAny16_6};  
WRITE8_7 	= {StartBit,OpCodeWRITE,AdrAny8_7};  
WRITE16_7 	= {StartBit,OpCodeWRITE,AdrAny16_7};   
WRITE8_8 	= {StartBit,OpCodeWRITE,AdrAny8_8};  
WRITE16_8 	= {StartBit,OpCodeWRITE,AdrAny16_8};  

WRAL8_4 	= {StartBit,OpCodeWRAL,AdrWRAL8_4};		// Write All Addresses 
WRAL16_4 	= {StartBit,OpCodeWRAL,AdrWRAL16_4};
WRAL8_5 	= {StartBit,OpCodeWRAL,AdrWRAL8_5};
WRAL16_5 	= {StartBit,OpCodeWRAL,AdrWRAL16_5};
WRAL8_6 	= {StartBit,OpCodeWRAL,AdrWRAL8_6};
WRAL16_6 	= {StartBit,OpCodeWRAL,AdrWRAL16_6};
WRAL8_7 	= {StartBit,OpCodeWRAL,AdrWRAL8_7};
WRAL16_7 	= {StartBit,OpCodeWRAL,AdrWRAL16_7};
WRAL8_8 	= {StartBit,OpCodeWRAL,AdrWRAL8_8};
WRAL16_8 	= {StartBit,OpCodeWRAL,AdrWRAL16_8};

ERASE8_4 	= {StartBit,OpCodeERASE,AdrAny8_4};  		// Clear Address
ERASE16_4 	= {StartBit,OpCodeERASE,AdrAny16_4};  
ERASE8_5 	= {StartBit,OpCodeERASE,AdrAny8_5};  
ERASE16_5 	= {StartBit,OpCodeERASE,AdrAny16_5};  
ERASE8_6 	= {StartBit,OpCodeERASE,AdrAny8_6};  
ERASE16_6 	= {StartBit,OpCodeERASE,AdrAny16_6};  
ERASE8_7 	= {StartBit,OpCodeERASE,AdrAny8_7};  
ERASE16_7 	= {StartBit,OpCodeERASE,AdrAny16_7};  
ERASE8_8 	= {StartBit,OpCodeERASE,AdrAny8_8};  
ERASE16_8 	= {StartBit,OpCodeERASE,AdrAny16_8};  

ERAL8_4 	= {StartBit,OpCodeERAL,AdrERAL8_4};		// Clear All Addresses 
ERAL16_4 	= {StartBit,OpCodeERAL,AdrERAL16_4};
ERAL8_5 	= {StartBit,OpCodeERAL,AdrERAL8_5};
ERAL16_5 	= {StartBit,OpCodeERAL,AdrERAL16_5};
ERAL8_6 	= {StartBit,OpCodeERAL,AdrERAL8_6};
ERAL16_6 	= {StartBit,OpCodeERAL,AdrERAL16_6};
ERAL8_7 	= {StartBit,OpCodeERAL,AdrERAL8_7};
ERAL16_7 	= {StartBit,OpCodeERAL,AdrERAL16_7};
ERAL8_8 	= {StartBit,OpCodeERAL,AdrERAL8_8};
ERAL16_8 	= {StartBit,OpCodeERAL,AdrERAL16_8};

$display("********************************************************");

	`ifdef cat93c46
		`ifdef X8
			`define		lgCom	10
			lgMem = 128;
			bufEWEN[9:0] 	= EWEN8_4[9:0];
			bufEWDS[9:0] 	= EWDS8_4; 
			bufREAD[9:0] 	= READ8_4; 
			bufWRITE[9:0] 	= WRITE8_4; 
			bufWRAL[9:0] 	= WRAL8_4;
			bufERASE[9:0] 	= ERASE8_4;
			bufERAL[9:0] 	= ERAL8_4;
			ORG = 0;
		`endif
		`ifdef X16
			`define		lgCom	9
			lgMem = 64;
			bufEWEN[8:0] = EWEN16_4;
			bufEWDS[8:0] 	= EWDS16_4; 
			bufREAD[8:0] 	= READ16_4; 
			bufWRITE[8:0] 	= WRITE16_4; 
			bufWRAL[8:0] 	= WRAL16_4;
			bufERASE[8:0] 	= ERASE16_4;
			bufERAL[8:0] 	= ERAL16_4;
			ORG = 1;
		`endif
	`endif
	`ifdef cat93hc46
		`define SeqRead	1
		`ifdef X8
			`define		lgCom	10
			lgMem = 128;
			bufEWEN[9:0] 	= EWEN8_4[9:0];
			bufEWDS[9:0] 	= EWDS8_4; 
			bufREAD[9:0] 	= READ8_4; 
			bufWRITE[9:0] 	= WRITE8_4; 
			bufWRAL[9:0] 	= WRAL8_4;
			bufERASE[9:0] 	= ERASE8_4;
			bufERAL[9:0] 	= ERAL8_4;
			ORG = 0;
		`endif
		`ifdef X16
			`define		lgCom	9
			lgMem = 64;
			bufEWEN[8:0] = EWEN16_4;
			bufEWDS[8:0] 	= EWDS16_4; 
			bufREAD[8:0] 	= READ16_4; 
			bufWRITE[8:0] 	= WRITE16_4; 
			bufWRAL[8:0] 	= WRAL16_4;
			bufERASE[8:0] 	= ERASE16_4;
			bufERAL[8:0] 	= ERAL16_4;
			ORG = 1;
		`endif
	`endif

	`ifdef cat93c56
		`define SeqRead	1
		`ifdef X8
			`define		lgCom	12
			lgMem = 256;
			bufEWEN[11:0] 	= EWEN8_5;
			bufEWDS[11:0] 	= EWDS8_5; 
			bufREAD[11:0] 	= READ8_5; 
			bufWRITE[11:0] = WRITE8_5; 
			bufWRAL[11:0] 	= WRAL8_5;
			bufERASE[11:0] = ERASE8_5;
			bufERAL[11:0] 	= ERAL8_5;
			ORG = 0;
		`endif
		`ifdef X16
			`define		lgCom	11
			lgMem = 128;
			bufEWEN[10:0] 	= EWEN16_5;
			bufEWDS[10:0] 	= EWDS16_5; 
			bufREAD[10:0] 	= READ16_5; 
			bufWRITE[10:0] = WRITE16_5; 
			bufWRAL[10:0] 	= WRAL16_5;
			bufERASE[10:0] = ERASE16_5;
			bufERAL[10:0] 	= ERAL16_5;
			ORG = 1;
		`endif
	`endif

	`ifdef cat93c57
		`define SeqRead	1
		`ifdef X8
			`define		lgCom	11
			lgMem = 256;
			bufEWEN[10:0] 	= EWEN8_7;
			bufEWDS[10:0] 	= EWDS8_7; 
			bufREAD[10:0] 	= READ8_7; 
			bufWRITE[10:0] = WRITE8_7; 
			bufWRAL[10:0] 	= WRAL8_7;
			bufERASE[10:0] = ERASE8_7;
			bufERAL[10:0] 	= ERAL8_7;
			ORG = 0;
		`endif
		`ifdef X16
			`define		lgCom	10
			lgMem = 128;
			bufEWEN[9:0] 	= EWEN16_7;
			bufEWDS[9:0] 	= EWDS16_7; 
			bufREAD[9:0] 	= READ16_7; 
			bufWRITE[9:0] = WRITE16_7; 
			bufWRAL[9:0] 	= WRAL16_7;
			bufERASE[9:0] = ERASE16_7;
			bufERAL[9:0] 	= ERAL16_7;
			ORG = 1;
		`endif
	`endif

	`ifdef cat93c66
		`define SeqRead	1
		`ifdef X8
			`define		lgCom	12
			lgMem = 512;
			bufEWEN[11:0] 	= EWEN8_6;
			bufEWDS[11:0] 	= EWDS8_6; 
			bufREAD[11:0] 	= READ8_6; 
			bufWRITE[11:0] = WRITE8_6; 
			bufWRAL[11:0] 	= WRAL8_6;
			bufERASE[11:0] = ERASE8_6;
			bufERAL[11:0] 	= ERAL8_6;
			ORG = 0;
		`endif
		`ifdef X16
			`define		lgCom	11
			lgMem = 256;
			bufEWEN[10:0] 	= EWEN16_6;
			bufEWDS[10:0] 	= EWDS16_6; 
			bufREAD[10:0] 	= READ16_6; 
			bufWRITE[10:0] = WRITE16_6; 
			bufWRAL[10:0] 	= WRAL16_6;
			bufERASE[10:0] = ERASE16_6;
			bufERAL[10:0] 	= ERAL16_6;
			ORG = 1;
		`endif
	`endif

	`ifdef cat93c76
		`define SeqRead	1
		`ifdef X8
			`define		lgCom	14
			lgMem = 1024;
			bufEWEN[13:0] 	= EWEN8_8;
			bufEWDS[13:0] 	= EWDS8_8; 
			bufREAD[13:0] 	= READ8_8; 
			bufWRITE[13:0] = WRITE8_8; 
			bufWRAL[13:0] 	= WRAL8_8;
			bufERASE[13:0] = ERASE8_8;
			bufERAL[13:0] 	= ERAL8_8;
			ORG = 0;
		`endif
		`ifdef X16
			`define		lgCom	13
			lgMem = 512;
			bufEWEN[12:0] 	= EWEN16_8;
			bufEWDS[12:0] 	= EWDS16_8; 
			bufREAD[12:0] 	= READ16_8; 
			bufWRITE[12:0] = WRITE16_8; 
			bufWRAL[12:0] 	= WRAL16_8;
			bufERASE[12:0] = ERASE16_8;
			bufERAL[12:0] 	= ERAL16_8;
			ORG = 1;
		`endif
	`endif

	`ifdef cat93c86
		`define SeqRead	1
		`define PEset	1
		`ifdef X8
			`define		lgCom	14
			lgMem = 2048;
			bufEWEN[13:0] 	= EWEN8_8;
			bufEWDS[13:0] 	= EWDS8_8; 
			bufREAD[13:0] 	= READ8_8; 
			bufWRITE[13:0] = WRITE8_8; 
			bufWRAL[13:0] 	= WRAL8_8;
			bufERASE[13:0] = ERASE8_8;
			bufERAL[13:0] 	= ERAL8_8;
			ORG = 0;
		`endif
		`ifdef X16
			`define		lgCom	13
			lgMem = 1024;
			bufEWEN[12:0] 	= EWEN16_8;
			bufEWDS[12:0] 	= EWDS16_8; 
			bufREAD[12:0] 	= READ16_8; 
			bufWRITE[12:0] = WRITE16_8; 
			bufWRAL[12:0] 	= WRAL16_8;
			bufERASE[12:0] = ERASE16_8;
			bufERAL[12:0] 	= ERAL16_8;
			ORG = 1;
		`endif
	`endif

	manAddr = lgMem;
	manRegB = `lgCom;
	TestPhase = 1;
//
//	First step in test
//
	`include "InputIni.v"

	manRegB = TestPhase;
	manRegW = $time;
	$display;	
	$display("_______________________________________________________________");
	$display;
	#0 $display("Test nr. %d,  time: %d",manRegB,manRegW);
	//
   memAndDisplayCrtOp;

//
	TestPhase = 2;
//
	Mstate = `idle;
	MnextState = `idle;
	MbitCounter = 0;
	evalSeqLength;
//
	SI = 1'b0;
	CSB = `B0;
	SCK = `B0;
	#tDelayIni MbitCounter = 0;
	if(MbitCounter == 0)
		begin
		//$display("Master: sent instruction:         %b",MtempReg,$time);
		SI = MtempReg[`lgCom-1];
		#0 MtempReg = MtempReg << 1;
		#(tDIS-tCSS) CSB =`B1;
		Mstate = `sendingInstruction;
		MnextState = `sendingInstruction;
		end
	MbitCounter = MbitCounter+1;
	//#tCSS 
#190 SCK = `B1;

	forever
		begin
		#(2*tHALF*dutyFactor) SCK = ~SCK;
		#(2*tHALF*(1-dutyFactor)) SCK = ~SCK;		
		end
	end

initial
	begin
	#(tDelayIni+tCSS+tDIH) SI = 0;
	end

//
//######################### TASKS ####################################
//

task createIncremFillPattern;
	begin
	genDataCell = 4'b0000;
	`ifdef X8
	for(i=0;i<realLSeq;i=i+1)
		begin
		bufWriteData[i] = {pattern,genDataCell};
		if(i < 16)
			genDataCell = genDataCell+1;
		else
			genDataCell = genDataCell-1;
		end
	`endif
	`ifdef X16
	for(i=0;i<realLSeq;i=i+2)
		begin
		bufWriteData[i] = {pattern,genDataCell};
		bufWriteData[i+1] = {pattern,genDataCell};
		if(i < 32)
			genDataCell = genDataCell+1;
		else
			genDataCell = genDataCell-1;
		end
	`endif
	end
endtask

task memAndDisplayCrtOp;
	begin
	if(Minstruction == EWEN)
		begin
		MtempReg = bufEWEN;
		$display("EWEN (Write Enable)");
		end
	if(Minstruction == EWDS)
		begin
		MtempReg = bufEWDS;
		$display("EWDS (Write Disable)");
		end
	if(Minstruction == READ)
		begin
		MtempReg = bufREAD;
		MtempReg[`lgCom-4:0] = addr[`lgCom-4:0];
		$display("READ (Read at address)");
		end
	if(Minstruction == WRITE)
		begin
		MtempReg = bufWRITE;
		MtempReg[`lgCom-4:0] = addr[`lgCom-4:0];
		$display("WRITE (Write at address)");
		if(lSeq == 1)
			begin
			`ifdef X8
				$display("%d bits: %b %b %b %b",`lgCom+8,MtempReg[`lgCom-1],MtempReg[`lgCom-2:`lgCom-3],MtempReg[`lgCom-4:0],dataByte[7:0]);
			`endif
			`ifdef X16
				$display("%d bits: %b %b %b %b %b",`lgCom+16,MtempReg[`lgCom-1],MtempReg[`lgCom-2:`lgCom-3],MtempReg[`lgCom-4:0],dataBytes[15:8],dataBytes[7:0]);
			`endif
			end
		else
			begin
			`ifdef X8
				$display("%d bits: %b %b %b ********",`lgCom+8,MtempReg[`lgCom-1],MtempReg[`lgCom-2:`lgCom-3],MtempReg[`lgCom-4:0]);
			`endif
			`ifdef X16
				$display("%d bits: %b %b %b ******** ********",`lgCom+16,MtempReg[`lgCom-1],MtempReg[`lgCom-2:`lgCom-3],MtempReg[`lgCom-4:0]);
			`endif
			$display("      pattern for data generation: %hH",pattern);
			end
		end
	if(Minstruction == WRAL)
		begin
		MtempReg = bufWRAL;
		$display("WRAL (Write All Addresses)");
		`ifdef X8
			$display("%d bits: %b %b %b %b",`lgCom+8,MtempReg[`lgCom-1],MtempReg[`lgCom-2:`lgCom-3],MtempReg[`lgCom-4:0],dataByte[7:0]);
		`endif
		`ifdef X16
			$display("%d bits: %b %b %b %b %b",`lgCom+16,MtempReg[`lgCom-1],MtempReg[`lgCom-2:`lgCom-3],MtempReg[`lgCom-4:0],dataBytes[15:8],dataBytes[7:0]);
		`endif
		end
	if(Minstruction == ERASE)
		begin
		MtempReg = bufERASE;
		MtempReg[`lgCom-4:0] = addr[`lgCom-4:0];
		$display("ERASE (Clear at address)");
		end
	if(Minstruction == ERAL)
		begin
		MtempReg = bufERAL;
		$display("ERAL (Clear All Addresses)");
		end
	if(Minstruction != WRITE && Minstruction != WRAL)
		$display("%d bits: %b %b %b",`lgCom,MtempReg[`lgCom-1],MtempReg[`lgCom-2:`lgCom-3],MtempReg[`lgCom-4:0] );
	end
endtask

task setBufLgWrite;
	begin						
	`ifdef X8
		begin
		bufWriteData[0] = dataByte;
		seqCounter = 1;
		end
	`endif
		`ifdef X16
		begin
		bufWriteData[0] = dataBytes[15:8];
		bufWriteData[1] = dataBytes[7:0];
		seqCounter = 2;
		end
	`endif
	end
endtask

task initBufReadData;
	begin
	for(i=0;i<2048;i=i+1)
		bufReadData[i] = 8'hxx;	
	end
endtask

task evalSeqLength;
	begin
	`ifdef X8
		realLSeq = lSeq;
	`endif
	`ifdef X16
		realLSeq = 2*lSeq;
	`endif
	seqCounter = realLSeq;
	end
endtask

task displayRecData;
	begin
	`ifdef SeqRead
		recDataCounter = recDataCounter;
	`else
		`ifdef X8
			recDataCounter = 1;
		`endif
		`ifdef X16
			recDataCounter = 2;
		`endif
	`endif
	`ifdef X8
		$display("    Master :  %d bytes of received data:",recDataCounter);
	`endif
	`ifdef X16
		$display("    Master :  %d words of received data:",recDataCounter/2);
	`endif

	if(recDataCounter == 1)
		begin
		$display("-----------");
		lgDisp = 1;
		end
	else
		if(recDataCounter == 2)
			begin
			$display("------------------");
			lgDisp = 2;
			end
	    else
		begin
		$display("---------------------------------------------------------------------");

		`ifdef X8
		if(recDataCounter > lgMem)
			begin
			lgDisp = lgMem;
			end
		else
			lgDisp = recDataCounter;
		`endif
		`ifdef X16
		if(recDataCounter > lgMem*2)
			begin
			lgDisp = lgMem*2;
			end
		else
			lgDisp = recDataCounter;
		`endif

		end

	for(i=0;i<lgDisp+1;i=i+1)
		begin
		j=i%16;
		if(j == 0 && i>0)
			begin

			`ifdef X8
			manAddr = addr+i-16;
			`endif
			`ifdef X16
			manAddr = addr+i/2-8;
			`endif

			if(manAddr >= lgMem)
				manAddr = manAddr - lgMem;
			$display("%h:  %h  %h  %h  %h  %h  %h  %h  %h  %h  %h  %h  %h  %h  %h  %h  %h",manAddr,linestr[0],linestr[1],linestr[2],linestr[3],linestr[4],linestr[5],linestr[6],linestr[7],linestr[8],linestr[9],linestr[10],linestr[11],linestr[12],linestr[13],linestr[14],linestr[15]);
			linestr[j] = bufReadData[i];
			end
		else
			linestr[j] = bufReadData[i];
		end

		if(j>0)
			begin
			case(j)
			1:
				begin
				manAddr = addr+i-j-1;
				$display("%h:   %h",manAddr,linestr[0]);
				end
			2:
				begin
				manAddr = addr+i-j-1;
				$display("%h:   %h  %h",manAddr,linestr[0],linestr[1]);
				end
			default:
				i=i;
			endcase
			end
		if(recDataCounter == 1)
			$display("-----------");
		else
			if(recDataCounter == 2)
				$display("------------------");
			else
				$display("---------------------------------------------------------------------");
		// manRegW =  $time;
		// $display("current address: %h,  time: %d",crtMemAddr,manRegW);
		$display;
		$display;
	end
endtask

//
//####################### ALWAYS #############################
//
//
//	State machine for MASTER activity
//
always @(negedge SCK) 
	begin
	if($time>0)
	begin
	Mstate = MnextState;
	#0 case(Mstate)
		`sendingInstruction: 		
			begin
			#(2*tHALF*(1-dutyFactor)-tDIS)  if(MbitCounter < `lgCom+1) 
				begin
				MbitCounter = MbitCounter + 1;
				end
			if(MbitCounter == `lgCom) 
				begin
				SI = MtempReg[`lgCom-1];
				#(tDIS+tDIH-1) SI = 1'b0;

				case(Minstruction)
					EWEN:
						begin
						MnextState = `sendingStopStart;
						end
					EWDS:
						begin
						MnextState = `sendingStopStart;
						end
					ERAL:
						begin
						Mstate = `internalWrite;
						MnextState = `internalWrite;
						end
					ERASE:
						begin
						Mstate = `internalWrite;
						MnextState = `internalWrite;
						end
					READ:
						begin
						recDataCounter = 0;
						initBufReadData;
						MnextState = `receivingData;
						end					
					WRITE:
						begin
						if(lSeq == 1)
							setBufLgWrite;
						else
							createIncremFillPattern;
						MnextState = `sendingData;
						end
					WRAL:
						begin
						addr = 0;
						setBufLgWrite;
						lSeq = 1;
						MnextState = `sendingData;
						end
				endcase
				#(2*tHALF*dutyFactor-tDIH) MbitCounter = 0;
				end
			else
				begin
				SI = MtempReg[`lgCom-1];
				#(tDIS+tDIH) SI = 1'b0;
				#0 MtempReg = MtempReg << 1;
				end
			end

		`sendingData:		
			begin
			if(MbitCounter==0)
				begin
				#(2*tHALF*(1-dutyFactor)-tDIS) MbitCounter=1;
				end
			case(MbitCounter)
			1:
				begin
				if(lSeq ==1)
					begin

					`ifdef X8
						MtempReg = bufWriteData[0];
					`endif
					`ifdef X16
					if(seqCounter%2 == 0)
						MtempReg = bufWriteData[0];
					else
						MtempReg = bufWriteData[1];
					`endif

					end
				else
					begin
					MtempReg[7:0] = bufWriteData[realLSeq-seqCounter];
					end
				SI = MtempReg[7];
				//$display("    Master: sent data %d: 		%h",realLSeq-seqCounter+1,MtempReg[7:0]);
				MtempReg = MtempReg << 1;
				#(tDIS+tDIH) SI = 0;
				#(2*tHALF-tDIS-tDIH) MbitCounter = MbitCounter + 1;
				SI = MtempReg[7];
				MtempReg = MtempReg << 1;
				#(tDIS+tDIH) SI = 0;
				end
			default:
				begin
				#(2*tHALF*(1-dutyFactor)-tDIS) SI = MtempReg[7];
				if(MbitCounter < `lgCom) 
					MbitCounter = MbitCounter + 1;
				#(tDIS+tDIH) SI = 0;
				MtempReg = MtempReg << 1;
				if(MbitCounter==8)
					begin
					seqCounter = seqCounter - 1;
					if(seqCounter == 0)
						begin	
						MbitCounter = 0;
						Mstate = `internalWrite;
						MnextState = `internalWrite;
						#(2*tHALF*dutyFactor-tDIH-delta) realLSeq = realLSeq;
						end
					else
						begin
						#(2*tHALF-tDIS-tDIH) MbitCounter = 1;
						if(Minstruction == WRAL)
							begin
								`ifdef X8
									MtempReg = bufWriteData[0];
								`endif

								`ifdef X16
									if(seqCounter%2 == 0)
										MtempReg = bufWriteData[0];
									else
										MtempReg = bufWriteData[1];
								`endif
							end

						if(Minstruction == WRITE)
							MtempReg = bufWriteData[realLSeq-seqCounter];
						//$display("    Master: sent data %d:           %h",realLSeq-seqCounter+1,MtempReg[7:0]);
						SI = MtempReg[7];
						#0 MtempReg = MtempReg << 1;
						#(tDIS+tDIH) SI = 0;
						#(2*tHALF-tDIS-tDIH) MbitCounter = MbitCounter + 1;
						SI = MtempReg[7];
						MtempReg = MtempReg << 1;
						#(tDIS+tDIH) SI = 0;
						end
					end
				end
			endcase
			end

		`receivingData:		
			begin
			case(MbitCounter)
			1:
				begin
				MtempReg[7:0] = 8'b00000000;
				#0 MtempReg[0] = SO;
				#(2*tHALF*(1-dutyFactor)) MbitCounter = MbitCounter + 1;
				end
			8:
				begin
				MtempReg = MtempReg << 1;
				MtempReg[0] = SO;
				seqCounter = seqCounter - 1;
				if(seqCounter == 0)
					begin
					bufReadData[recDataCounter] = MtempReg[7:0];
					recDataCounter = recDataCounter + 1;
					displayRecData;
					MnextState = `sendingStopStart;
					#(2*tHALF*(1-dutyFactor)) MbitCounter = 0; 
				//	$display("Master: rec. data %d: 		%h",lSeq-seqCounter,MtempReg[7:0]);
					end
				else
					begin
					bufReadData[recDataCounter] = MtempReg[7:0];
					recDataCounter = recDataCounter + 1;
					MnextState = `receivingData;
					#(2*tHALF*(1-dutyFactor)/2) MbitCounter = 1; 
					end
				end
			// cases 2,3,4,5,6,7
			default:
				begin
				MtempReg = MtempReg << 1;
				#0 MtempReg[0] = SO;
				#(2*tHALF*(1-dutyFactor)) MbitCounter = MbitCounter + 1;
				end
			endcase
			end

		`internalWrite:	
			begin
			#tCSH CSB = 0;
			#tCSMIN CSB = 1;			// Select device (CSB high) && polling
			#(tSV+10) $display("    Master:         Begin polling... ",$time);
			#0 while(SO != 1'b1)
				#100 SI = SI;
			$display("    Master:         .....end polling ",$time);
			#delta CSB = `B0;
			Mstate = `sendingStopStart;
			MnextState = `sendingStopStart;
			end

		`sendingStopStart:	
			begin
				case(TestPhase)
				`include "Input.v"
				//`include "c:/Modeltech_xe/TestMicrowire/Input.v"
				default:
				$stop;
				endcase
				manRegB = TestPhase;
				manRegW = $time;
				$display;
				$display("_______________________________________________________________");
				$display;
				$display("Test nr. %d,  time: %d",manRegB,manRegW); 
				$display;
				memAndDisplayCrtOp;
				TestPhase = TestPhase + 1;

			evalSeqLength;

			#0 case(Minstruction)
			EWEN:
				begin
				$display;
				end
			ERAL:
				begin
				$display;
				end
			EWDS:
				begin
				$display;
				end
			WRAL:
				begin
				`ifdef X8
					$display("         data:            %hH",dataByte);
				`endif
				`ifdef X16
					$display("         data:            %hH",dataBytes);
				`endif
				$display;
				end
			READ:
				begin
				$display("         address:         %h",addr);
				manRegB = lSeq;

				`ifdef X8
					$display("         length :         %d bytes",manRegB);
				`endif
				`ifdef X16
					$display("         length :         %d words",manRegB);
				`endif

				$display;
				end
			ERASE:
				begin
				$display("         address:         %h",addr);
				$display;
				end
			WRITE:
				begin
				$display("         address:         %h",addr);
				if(lSeq == 1)
					begin
					`ifdef X8
						$display("         length: %d bytes",lSeq);
						$display("         data:            %hH",dataByte);
					`endif
					`ifdef X16
						$display("         length: %d words",lSeq);
						$display("         data:            %hH",dataBytes);
					`endif
					end
				else
					begin
					`ifdef X8
						$display("         length: %d bytes",lSeq);
					`endif
					`ifdef X16
						$display("         length: %d words",lSeq);
					`endif
					end
				$display;

				end
			endcase
			// STOP

			#tCSH CSB = 0;	
			//$display("####### Master: CSB = 0 ",$time);
			#tCSMIN CSB =`B1;		//START
			//$display("####### Master: CSB = 1 ",$time);
			//$display("    Master: sending START at time ",$time);
			Mstate = `sendingInstruction;
			MnextState = `sendingInstruction;
			if(2*tHALF*(1-dutyFactor)-tCSH-tCSMIN-tDIS >= 0)
				begin
				#(2*tHALF*(1-dutyFactor)-tCSH-tCSMIN-tDIS) if(MbitCounter == 0)
					begin
					SI = MtempReg[`lgCom-1];
					#0 MtempReg = MtempReg << 1;
					end
				// $display("Master: sent instruction:         %b",MtempReg);
				MbitCounter = MbitCounter+1;
				#(tDIS+tDIH) SI=0;
				end
			end
	endcase 
	end 
	end

endmodule

